@@ -29,7 +29,7 @@ class Phase(str, _Enum):
     TRAIN = 'train'
     VALIDATION = 'validation'
     TEST = 'test'
-    EVAL = 'eval'
+    INFERENCE = 'inference'
 
 
 class LogKey(str, _Enum):
@@ -42,7 +42,3 @@ class LogKey(str, _Enum):
 
     TEST_METRICS = 'test_metrics'
     SERIALIZABLE_TEST = 'serializable_test_scores'
-
-    GLOBAL_TEST_LOG = 'global_test_log'
-    GLOBAL_TEST_METRICS = 'global_test_metrics'
-    SERIALIZABLE_GLOBAL_TEST = 'serializable_global_test_scores'
